@@ -386,6 +386,16 @@
 #   define S_ISREG(m) (((m) & S_IFREG) != 0)
 #endif
 
+#if defined (_MAX_PATH) && !defined (PATH_MAX)
+#   define PATH_MAX _MAX_PATH
+#endif
+#if !defined (PATH_MAX)
+#   define PATH_MAX 1024
+#endif
+
+#if defined (WIN32)
+#   define getcwd _getcwd
+#endif
 
 //- Check compiler data type sizes ------------------------------------------
 
@@ -525,8 +535,10 @@ typedef struct {
 #   if (!defined (__cplusplus) && (!defined (inline)))
 #       define inline __inline
 #   endif
-#   define strtoull _strtoui64
-#   define atoll _atoi64
+#   if (!defined (__MINGW32__))
+#     define strtoull _strtoui64
+#     define atoll _atoi64
+#   endif
 #   define srandom srand
 #   define TIMEZONE _timezone
 #   if (!defined (__MINGW32__))
@@ -683,6 +695,9 @@ char *if_indextoname (unsigned int ifindex, char *ifname);
 #if defined (__UTYPE_OSX) && !defined (HAVE_UUID)
 #   define HAVE_UUID 1
 #endif
+#if defined (__UTYPE_FREEBSD) && !defined (HAVE_UUID)
+#   define HAVE_UUID 1
+#endif
 #if defined (HAVE_UUID)
 #   if defined (__UTYPE_FREEBSD) || defined (__UTYPE_NETBSD) || defined(__UTYPE_OPENBSD)
 #       include <uuid.h>
