@@ -28,7 +28,7 @@
 
 //  zframe_t instances always have this tag as the first 4 octets of
 //  their data, which lets us do runtime object typing & validation.
-#define ZFRAME_TAG              0x0002cafe
+#define ZFRAME_TAG              0xcafe0002
 
 //  Structure of our class
 
@@ -409,7 +409,7 @@ zframe_set_group (zframe_t *self, const char *group)
     strcpy (self->group, group);
     return 0;
 #else
-    errno = EINVAL;
+    errno = ENOTSUP;
     return -1;
 #endif
 }
@@ -745,6 +745,12 @@ zframe_test (bool verbose)
 
     zsock_destroy (&dish);
     zsock_destroy (&radio);
+#else
+    frame = zframe_new ("Hello", 5);
+    rc = zframe_set_group (frame, "World");
+    assert(rc == -1);
+    assert(errno == ENOTSUP);
+    zframe_destroy (&frame);
 #endif
 
 #if defined (__WINDOWS__)
